/* Colocs - long range heteronuclear correlation experiment

   1993mar01, R.Kyburz - streamlined, changed for phase tables
   2014feb04, B.Heise  - Chempack compatible, VnmrJ 3+ compatible, shorter phase cycle, Redfield trick added
*/

#include <standard.h> 
#include <chempack.h> 

/* original phase cycle - minimum of 16...
static int phi1[2]  = {1,3},
           phi2[2]  = {2,0},
           phi3[4]  = {0,1,2,3},
           phi4[8]  = {0,2,0,2,2,0,2,0},
           phi5[16] = {0,0,0,0,1,1,1,1,2,2,2,2,3,3,3,3};
*/

// newer phase cycle, works with min. 4 scans
static int phi1[2]  = {1,3},
           phi2[2]  = {2,0},
           phi3[8]  = {0,1,0,1,2,3,2,3},
           phi4[16] = {0,2,2,0,0,2,2,0,2,0,0,2,2,0,0,2},
           phi5[16] = {0,0,1,1,2,2,3,3,1,1,0,0,3,3,2,2};

pulsesequence()
{
   double	pp     = getval("pp"),
                pplvl  = getval("pplvl"),
                tau    = getval("tau"),
                tau3   = getval("tau3"),
                tau4   = getval("tau4"),
		j1xh   = getval("j1xh"),
		jnxh   = getval("jnxh"),
                hsglvl = getval("hsglvl"),
                hsgt   = getval("hsgt"),
                satdly = getval("satdly"),
                nimax;
   char         sspul[MAXSTR],coupled[MAXSTR],colocs[MAXSTR],
                satmode[MAXSTR];

   getstr("colocs",colocs);
   getstr("coupled", coupled);
   getstr("sspul",sspul);
   getstr("satmode",satmode);

/* CALCULATE PHASE CYCLE */
  settable(t1, 2, phi1);
  settable(t2, 2, phi2);
  settable(t3, 8, phi3);
  settable(t4,16, phi4);
  settable(t5,16, phi5);

  assign(ct,v1);
  getelem(t1,v1,v2);
  getelem(t4,v1,oph);

  initval(2.0*(double)(((int)(d2*getval("sw1")+0.5)%2)),v5);
  add(v2,v5,v2);
  add(oph,v5,oph);


/* safety checks */
   dm[A]  = 'n'; dm[B]  = 'n';
   dmm[A] = 'c'; dmm[B] = 'c';
   if ((jnxh <= 0.1) && ((tau3 == 0.0) || (tau4 == 0.0)))
     { printf("tau3/tau4 or jnxh must be set first");
       psg_abort(1); }

/* calculation of delays, if necessary */
   if (tau == 0.0)
   {
      if (j1xh != 0.0) tau = 1.0/(2.0*j1xh)-7/3*pw;
      else             tau = 3.3e-3;
   }

   nimax = (double) ((int) (2.0*getval("sw1") * tau3 + 1.0));
   if (getval("ni") > nimax)
   {
      printf("the evolution time is too long; reduce ni to %5.0f\n",nimax);
      psg_abort(1);
   }

   status(A); 
   if (sspul[0] == 'y')
   {
     zgradpulse(hsglvl,hsgt);
     rgpulse(pw,zero,rof1,rof1);
     zgradpulse(hsglvl,hsgt);
    }

    if (satmode[0] == 'y')
    {
      if (d1 - satdly > 0)
         delay(d1 - satdly);
      else
        delay(0.02);
        obspower(satpwr);
        if (satfrq != tof)
         obsoffset(satfrq);
        rgpulse(satdly,zero,rof1,rof1);
        if (satfrq != tof)
         obsoffset(tof);
       obspower(tpwr);
       delay(1.0e-5);
    }
   else
   {  delay(d1); }

   if (getflag("wet")) 
     wet4(zero,one);

status(B); 
      decpower(pplvl); 
      if (colocs[A] == 'y') 
      {            /* if colocs experiment do tango pulse */
         decrgpulse(pp/2.0,zero,rof1,rof1);
         delay(tau);
         rgpulse(pw, v2,rof1,rof1);
         simpulse(2.0*pw, 2.0*pp, t2, t1, rof1, rof1);
         rgpulse(pw, t1,rof1,rof1);
         delay(tau);
         decrgpulse(pp/2.0, two,rof1,rof1);
      }
      else         /* simple pulse for coloc */
         decrgpulse(pp, zero,rof1,rof1);
   
      if (coupled[A] == 'y') 
      {            /* Do not refocus carbons to see JCCH */
         if (d2/2.0-rof1 > 0) delay(d2/2.0 - rof1);
         decrgpulse(2.0*pp, t3,rof1,rof1);
         delay(tau3 - d2/2.0);
      }
      else
      {
         if (d2/2.0-rof1 > 0) delay(d2/2.0 - rof1);
         rgpulse(pw, t1,rof1,rof1);
         simpulse(240.0*pw/90.0, 2.0*pp, t2, t3, rof1, rof1);
         rgpulse(pw, t1,rof1,rof1);
         delay(tau3 - d2/2.0);
      }

      if (colocs[A] == 'y')
      {
         simpulse(pw, pp, t5, t5, rof1, rof1);
         if (tau4/2.0 - tau - rof1 > 0) delay(tau4/2.0 - tau - rof1);
         decrgpulse(pp, zero,rof1,rof1);
         delay(tau);
         rgpulse(pw, t1,rof1,rof1);
         simpulse(2.0*pw, 2.0*pp, t2, t1, rof1, rof1);
         rgpulse(pw, t1,rof1,rof1);
         delay(tau);
         decrgpulse(pp, zero,rof1,rof1);
         if (tau4/2.0 - tau > 0) delay(tau4/2.0 - tau);
      }
      else
      {
         simpulse(pw, pp, t5, t5, rof1, rof2);
         delay(tau4);
      }

      decpower(dpwr);
   status(C); 
}
